/*
 * two windows: one normal, one top-view
 * z-buffer show on off
 * double single buff
 *
 *
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <vector>

#include "misc.h"
#include "vecmath.h"
#include "image.h"
#include "reCamera.h"
#include "reTexture.h"
#include "miscOGL.h"

using namespace std;

enum {MENU_EXIT,MENU_TOGGLE_WIRE,MENU_TOGGLE_TEXTURE,MENU_TOGGLE_DEPTH,MENU_TOGGLE_SPLIT,MENU_TOGGLE_ANIMATE,MENU_TOGGLE_MIPMAP};

int mWinWidth=1024,mWinHeight=512;
Vec3f mCamPos,mLookAt,mUp;
int mPrevMouseX,mPrevMouseY,mMouseX,mMouseY;
bool mShiftDown,mCtrlDown,mAltDown;
bool mMouseDown,mPrevMouseDown;
int mMouseButton;

reTexture mTexture,mTexture2;
reTexture mTextureNear,mTexture2Near;

static reCamera mCamera;
static bool mWireframe=false;
static bool mTextureOn=false;
static bool mDepthOn=true;
static bool mSplitScreen=false;
static bool mAnimate=false;
static bool mMipmapOn=true;

void reshape(int width, int height)
{
	if(mSplitScreen)
	{
		mWinWidth=width;
		mWinHeight=height;
		glViewport(0, 0, mWinWidth/2, mWinHeight);
		mCamera.setResolution(mWinWidth/2, mWinHeight);
		mCamera.setOGLMatrices();
	}
	else
	{
		mWinWidth=width;
		mWinHeight=height;
		glViewport(0, 0, mWinWidth, mWinHeight);
		mCamera.setResolution(mWinWidth, mWinHeight);
		mCamera.setOGLMatrices();
	}
}

static GLUquadricObj *gQuad=NULL;
void renderSphere(float x,float y, float z)
{
	static int mDListNo=0;
	if(mDListNo==0)  // have we generated a display list already?
	{
		if(gQuad==NULL) gQuad=gluNewQuadric();  // make a better solution later
		// generate a display list for the sphere
		gluQuadricDrawStyle(gQuad, GLU_FILL);
		
		mDListNo=glGenLists(1);
		glNewList(mDListNo,GL_COMPILE);
		gluSphere(gQuad,0.2,10,10);
		glEndList();
	}
	glPushMatrix();
	glTranslatef(x,y,z);
	glCallList(mDListNo);
	glPopMatrix();
}

void setMtl(Vec3f ambient,Vec3f diffuse, Vec3f specular,float shininess)
{
	Vec4f amb(ambient[0], ambient[1], ambient[2], 1.0f);
	Vec4f diff(diffuse[0], diffuse[1], diffuse[2], 1.0f);
	Vec4f spec(specular[0], specular[1], specular[2], 1.0f);
	Vec4f black(0,0,0,1);
	
	glMaterialfv(GL_FRONT, GL_AMBIENT,  amb.vec);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,  diff.vec);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec.vec);
	glMaterialfv(GL_FRONT, GL_EMISSION, black.vec);
	glMaterialf(GL_FRONT, GL_SHININESS, shininess);		
}

void setLight(void)
{
	float L_amb[]={1,1,1,1};
	float L_diff[]={1,1,1,1};
	float L_pos[]={3,3,3,0};
	glLightfv(GL_LIGHT0, GL_DIFFUSE, L_diff);
	glLightfv(GL_LIGHT0, GL_AMBIENT, L_amb);
	glLightfv(GL_LIGHT0, GL_POSITION, L_pos);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glDisable(GL_LIGHTING);
	glColor3f(1,1,0);
	renderSphere(L_pos[XI],L_pos[YI],L_pos[ZI]);
	glEnable(GL_LIGHTING);
}

void drawQuadOpenGL(bool wireframe)
{
	if(mMipmapOn) mTexture.renderOGL();
	else mTextureNear.renderOGL();

	float s=10.0f*0.5f;
	glColor3f(1,1,1);
	if(!wireframe)
	{
		glBegin(GL_QUADS);
		glNormal3f(0,1,0);
		glTexCoord2f(0,0);
		glVertex3f(-s,0,-s);
		glTexCoord2f(0,10);
		glVertex3f(s,0,-s);
		glTexCoord2f(10,10);
		glVertex3f(s,0,s);
		glTexCoord2f(10,0);
		glVertex3f(-s,0,s);		
		glEnd();
	}
	else
	{
		glBegin(GL_LINE_LOOP);
		glNormal3f(0,1,0);
		glTexCoord2f(0,0);
		glVertex3f(-s,0,-s);
		glTexCoord2f(1,0);
		glVertex3f(s,0,-s);
		glTexCoord2f(1,1);
		glVertex3f(s,0,s);
		glEnd();
		
		glBegin(GL_LINE_LOOP);
		glNormal3f(0,1,0);
		glTexCoord2f(0,0);
		glVertex3f(-s,0,-s);
		glTexCoord2f(1,1);
		glVertex3f(s,0,s);
		glTexCoord2f(0,1);
		glVertex3f(-s,0,s);		
		glEnd();
	}
}



void renderTeapot(bool wireframe)
{
	if(mMipmapOn) mTexture2.renderOGL();
	else mTexture2Near.renderOGL();

	glFrontFace(GL_CW);
	glEnable(GL_AUTO_NORMAL);
	glEnable(GL_NORMALIZE);		
	if(wireframe) glutWireTeapot(1.0);
	else glutSolidTeapot(1.0);
	glFrontFace(GL_CCW);	
}



void renderScene(bool wireframe)
{
	setLight();
	glDisable(GL_LIGHTING);
	if(mTextureOn)
	{
		glEnable(GL_TEXTURE_2D);        
		glDisable(GL_LIGHTING);
	}
	else
	{
		glDisable(GL_TEXTURE_2D);        
		glEnable(GL_LIGHTING);
	}
	
	setMtl(Vec3f(0.2f,0.2f,0.2f),Vec3f(0.7f,0.2f,0.2f),Vec3f(0.4f,0.4f,0.4f),20);

	drawQuadOpenGL(wireframe);

	setMtl(Vec3f(0.2f,0.2f,0.2f),Vec3f(0.4f,0.4f,0.9f),Vec3f(0.8f,0.8f,0),20);	
	glPushMatrix();
	glTranslatef(0,0.75,0);
	if(mAnimate)
	{
		static float time=0.0f;
		time+=0.02f;
		glTranslatef(0,1.5*fabs(cos(time)),0);		
		glutPostRedisplay();
	}
	renderTeapot(wireframe);
	glPopMatrix();	
}



void display(void)
{
	glClearColor(1,1,1,1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	
	reshape(mWinWidth,mWinHeight);
	
	renderScene(mWireframe);

	if(mSplitScreen)
	{
		reCamera camera;
		camera.setResolution(mWinWidth/2,mWinHeight);
		camera.setPosition(Vec3f(0,30,0));
		camera.setLookAt(Vec3f(0,0,0));
		camera.setUpVector(Vec3f(0,0,-1));
		camera.setNearFar(1,100);
		glViewport(mWinWidth/2, 0, mWinWidth/2, mWinHeight);
		camera.setOGLMatrices();

		renderScene(mWireframe);
		mCamera.renderOGL();
		setMtl(Vec3f(0.2f,0.2f,0.2f),Vec3f(0.4f,0.4f,0.4f),Vec3f(0.4f,0.4f,0.4f),20);
		renderSphere(mCamera.getPosition()[XI],mCamera.getPosition()[YI],mCamera.getPosition()[ZI]);
	}
	

	
	glutSwapBuffers();
	glutPostRedisplay();
}

void moveCamera(Vec3f &campos,Vec3f &lookat,Vec3f &camup,bool shift,bool ctrl, bool alt,
				int mouseX,int mouseY,int prevMouseX,int prevMouseY)
{
	Mtx3f mtx,mtx2;
	Vec3f tmpvec,tmpright,tmpup;
	Vec3f camdir,camright;
	camdir=lookat-campos;
	camdir.normalize();
	if(shift)
	{
		float delta;
		delta=(mouseY-prevMouseY)*0.04;
		delta*=delta;
		if(mouseY-prevMouseY<0) delta=-delta;
		campos+=camdir*delta;
		glutPostRedisplay();
	}
	else if(ctrl)
	{
		camright=camdir%camup;
		camright.normalize();
      
		float delta;
		delta=(mouseY-prevMouseY)*0.04;
		delta*=delta;
		if(mouseY-prevMouseY<0) delta=-delta;
		campos+=camup*delta;
		lookat+=camup*delta;
		delta=(mouseX-prevMouseX)*0.04;
		delta*=delta;
		if(mouseX-prevMouseX<0) delta=-delta;
		campos+=camright*delta;
		lookat+=camright*delta;
		glutPostRedisplay();
	}
	else
	{
      // rotate real camera (always looks at (0,0,0))
      // first rotate around up-axis
      // a really simpe navigator
  
      real rad=(prevMouseX-mouseX)*0.01;
      camright=camdir%camup;
      camright.normalize();
      
      mtx.rotAxis(camup,rad);
      tmpvec=mtx*campos;
      tmpright=mtx*camright;
      
      rad=-(mPrevMouseY-mMouseY)*0.01;
      mtx2.rotAxis(tmpright,rad);
      campos=mtx2*tmpvec;
      camright=tmpright;
      tmpup=mtx2*camup;
      camup=tmpup;
      glutPostRedisplay();
   }
}


void recordMotion(int x, int y)
{
	mPrevMouseDown=mMouseDown;
	mMouseX=x;
	mMouseY=mWinHeight-y-1;
	if(mMouseButton==GLUT_LEFT_BUTTON)
	{
		moveCamera(mCamPos,mLookAt,mUp,mShiftDown,mCtrlDown,mAltDown,mMouseX,mMouseY,mPrevMouseX,mPrevMouseY);
		glutPostRedisplay();
	}
	mCamera.setPosition(mCamPos);
	mCamera.setLookAt(mLookAt);
	mCamera.setUpVector(mUp);
	mPrevMouseX=mMouseX;
	mPrevMouseY=mMouseY;
}

void recordMouse(int button, int state, int x, int y)
{
	mShiftDown=glutGetModifiers() & GLUT_ACTIVE_SHIFT ? true : false;
	mCtrlDown=glutGetModifiers() & GLUT_ACTIVE_CTRL ? true : false;
	mAltDown=glutGetModifiers() & GLUT_ACTIVE_ALT ? true : false;

	mPrevMouseDown=mMouseDown;
	mMouseDown=state==GLUT_UP ? false : true;
	mMouseX=x;
	mMouseY=mWinHeight-y-1;
	mPrevMouseX=mMouseX;
	mPrevMouseY=mMouseY;
	mMouseButton=button;	 // GLUT_LEFT_BUTTON, GLUT_MIDDLE_BUTTON, or GLUT_RIGHT_BUTTON
}


void keyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
	case 27:
		exit(0);  // ugly exit
		break;
	case 'w':
	case 'W':
		mWireframe=!mWireframe;
	case 'a':
	case 'A':
		mAnimate=!mAnimate;
		break;
	case 't':
	case 'T':
		mTextureOn=!mTextureOn;
		break;
	case 'm':
	case 'M':
		mMipmapOn=!mMipmapOn;
		break;
	case 'z':
	case 'Z':
		mDepthOn=!mDepthOn;
		if(mDepthOn) glDepthFunc(GL_LEQUAL);
		else glDepthFunc(GL_ALWAYS);
		break;
	case 's':
	case 'S':
		mSplitScreen=!mSplitScreen;
		reshape(mWinWidth,mWinHeight);
		break;		
	}
	glutPostRedisplay();
}

void menus(int value)
{
	switch(value)
	{
	case MENU_TOGGLE_WIRE:
		mWireframe=!mWireframe;
		break;
	case MENU_TOGGLE_TEXTURE:
		mTextureOn=!mTextureOn;
		break;
	case MENU_TOGGLE_ANIMATE:
		mAnimate=!mAnimate;
		break;
	case MENU_TOGGLE_DEPTH:
		mDepthOn=!mDepthOn;
		if(mDepthOn) glDepthFunc(GL_LEQUAL);
		else glDepthFunc(GL_ALWAYS);
		break;
	case MENU_TOGGLE_MIPMAP:
		mMipmapOn=!mMipmapOn;
		break;
	case MENU_TOGGLE_SPLIT:
		mSplitScreen=!mSplitScreen;
		reshape(mWinWidth,mWinHeight);
		break;
	case MENU_EXIT:
		exit(0);  // ugly exit
		break;
	}
	glutPostRedisplay();
}

void init(void)
{	
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	mCamera.setResolution(mWinWidth,mWinHeight);
	mCamera.setPosition(Vec3f(4,1,5));
	mCamera.setLookAt(Vec3f(0,0,0));
	mCamera.setUpVector(Vec3f(0,1,0));
	mCamera.setNearFar(1,20);
	mMouseButton=GLUT_LEFT_BUTTON;
	mMouseDown=false;
	mPrevMouseDown=false;
	mMouseX=0;
	mMouseY=0;
	mPrevMouseX=0;
	mPrevMouseY=0;
	mUp=mCamera.getUpVector();
	mCamPos=mCamera.getPosition();
	mLookAt=mCamera.getLookAt();

	// read texture
	int w,h;
	unsigned char *pixels;
	if(fReadPPM("3d.ppm", w, h, pixels))
//	if(fReadPPM("checker.ppm", w, h, pixels))
//	if(fReadPPM("random.ppm", w, h, pixels))
	{
		printf("Read texure OK\n");
		mTexture.set("test",w,h,pixels,true);
		mTextureNear.set("test",w,h,pixels,false);
	}
	else
	{
		printf("Error: could not read texture\n");
	}
	
	if(fReadPPM("checker.ppm", w, h, pixels))
	{
		printf("Read texure OK\n");
		mTexture2.set("checker",w,h,pixels,true);
		mTexture2Near.set("checker",w,h,pixels,false);
	}
	else
	{
		printf("Error: could not read texture\n");
	}
}

int main(int argc, char* argv[])
{
	glutInit(&argc, argv);
    glutInitWindowPosition(200,200);
	glutInitWindowSize(mWinWidth,mWinHeight);
	glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
	glutCreateWindow("EDA 425 Demo");
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutReshapeFunc(reshape);
	glutMouseFunc(recordMouse);
	glutMotionFunc(recordMotion);
	//glutSpecialFunc(special);
	glutCreateMenu(menus);
	glutAddMenuEntry("[A] Toggle Animation", MENU_TOGGLE_ANIMATE);
	glutAddMenuEntry("[W] Toggle Wireframe", MENU_TOGGLE_WIRE);
	glutAddMenuEntry("[T] Toggle Textures", MENU_TOGGLE_TEXTURE);
	glutAddMenuEntry("[Z] Toggle Depth Testing", MENU_TOGGLE_DEPTH);
	glutAddMenuEntry("[S] Toggle Split Screen", MENU_TOGGLE_SPLIT);
	glutAddMenuEntry("[M] Toggle MIPMAP/NEAREST", MENU_TOGGLE_MIPMAP);
	glutAddMenuEntry("[ESC] Exit", MENU_EXIT);
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	
	init();

	glutMainLoop();
	return 0;       
}
