// -------------------------------------------------------------------------
// File:    miscOGL.cxx
// Desc:    a collection of useful OpenGL related functions and stuff
//
// Author:  Tomas Mller
// History: June, 2000 (started)
// -------------------------------------------------------------------------

#include "miscOGL.h"

void fColorModulateImage(int width, int height, GLubyte *color)
{
   glPushAttrib(GL_COLOR_BUFFER_BIT);
   {
      glDisable(GL_DITHER);
      glEnable(GL_BLEND);
      // dest=destcolor*srccolor + 0*destcolor
      glBlendFunc(GL_DST_COLOR, GL_ZERO);       
      fDrawColorBuffer(width,height,color,GL_RGB,GL_BACK);
   }
  glPopAttrib();  
}

void fLuminanceModulateImage(int width, int height, GLubyte *color)
{
   glPushAttrib(GL_COLOR_BUFFER_BIT);
   {
      glDisable(GL_DITHER);
      glEnable(GL_BLEND);
      // dest=destcolor*srccolor + 0*destcolor
      glBlendFunc(GL_DST_COLOR, GL_ZERO);       
      fDrawColorBuffer(width,height,color,GL_LUMINANCE,GL_BACK);
   }
  glPopAttrib();  
}

// ImageComponent can be GL_LUMINANCE, GL_RGB etc
// DrawBuffer can be GL_BACK, GL_FRONT, etc
void fDrawColorBuffer(int width, int height, GLubyte *color,
		      const GLint ImageComponent, const GLint DrawBuffer)
{
  GLint UnpackAlignment;
  glGetIntegerv(GL_UNPACK_ALIGNMENT,&UnpackAlignment); 
  glPixelStorei(GL_UNPACK_ALIGNMENT,1); 

  GLint OldDrawBuffer;
  glGetIntegerv(GL_DRAW_BUFFER,&OldDrawBuffer);
  glDrawBuffer(DrawBuffer);

  glPushAttrib(GL_LIGHTING_BIT);
  glDisable(GL_LIGHTING);
  glDisable(GL_COLOR_MATERIAL);
  
  glPushAttrib(GL_DEPTH_BUFFER_BIT);
  glDisable(GL_DEPTH_TEST);
  glDepthMask(0);
  
  glPushAttrib(GL_TEXTURE_BIT);
  glDisable(GL_TEXTURE_2D);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0, glutGet((GLenum)GLUT_WINDOW_WIDTH) - 1, 
	     0, glutGet((GLenum)GLUT_WINDOW_HEIGHT) - 1);

  glRasterPos2i(0,0);
  glDrawPixels(width,height, ImageComponent, GL_UNSIGNED_BYTE, color);

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
 
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

  glPopAttrib();
  glPopAttrib();
  glPopAttrib();

  glDrawBuffer(OldDrawBuffer);
  glPixelStorei(GL_UNPACK_ALIGNMENT,UnpackAlignment); 
}

void fPushOGLCamera(void)
{
   // push state
   glPushAttrib(GL_VIEWPORT_BIT);
 
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glMatrixMode(GL_PROJECTION);
   glPushMatrix(); 
}

void fPopOGLCamera(void)
{
   // pop state
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();

   glPopAttrib();
}
