// -------------------------------------------------------------------------
// File:    reCamera.h 
// Desc:    base class for a camera (position, view direction, 
//          upvec, FOV, viewport, etc)
//
// Author:  Tomas Mller
// History: March, 2000 (started)
// -------------------------------------------------------------------------

#ifndef RE_CAMERA_H
#define RE_CAMERA_H

#include "vecmath.h"

class reCamera
{
 public:

            reCamera();
   virtual ~reCamera();

   void  setPosition( const Vec3f );
   void  setUpVector(const Vec3r up);
   void  setLookAt(const Vec3r lookat);
   void  setVFOV(real vfov);
   void  setResolution(int width,int height);
   void  setNearFar(real n,real f);
   void  getNearFar(real &near_value,real &far_value) const;
   real  getNear(void) const;
   real  getFar(void) const;
   Vec3r getPosition(void) const;
   Vec3r getDirection(void) const;
   Vec3r getUpVector(void) const;
   Vec3r getLookAt(void) const;
   real  getVFOV(void) const;
   int   getWidth(void) const;
   int   getHeight(void) const;
   void  renderOGL(void) const;
   void  getTracerParams(Vec3r &campos, Vec3r &lower_left,Vec3r &deltax,Vec3r &deltay);
   void  setOGLMatrices(void) const;
   void  debugprint(void);

 protected:
   Vec3r mPosition;
   Vec3r mDirection;  // view direction
   Vec3r mUpVector;
   Vec3r mLookAt;     // this one is the best to give
   real  mVFOV;       // vertical FOV (as in OpenGL)   
   int   mWidth;
   int   mHeight;
   bool  mPreComputed;  // true if the following parameters are valid:
   Vec3r mLowerLeft;   // the center of the lower left pixel on the screen
   Vec3r mDeltaX;      // delta vector for one pixel in x-dir
   Vec3r mDeltaY;      // delta vector for one pixel in y-dir
   real  mNear,mFar;
   void  precomputeTracerParams(void);
};

#endif
