// -------------------------------------------------------------------------
// File:    reTexture.cxx
// Desc:    a texture
//
// Author:  Tomas Mller
// History: June, 2000 (started)
// -------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <GL/glut.h>

#include "misc.h"
#include "reTexture.h"

reTexture::reTexture()
{
   mWidth=0;
   mHeight=0;
   mPixels=NULL;
   mName=NULL;
   mOGLTextureNum=0;  // i.e., unvalid texture
	mMipmap=true;
   /*  fprintf(stderr, "New texture = %d\n", this); */
}

reTexture::~reTexture()
{
   if(mPixels) delete [] mPixels;
}

void reTexture::set(char *name,int width, int height, unsigned char *pixels,bool mipmap)
{
   ASSERT(width>0 && height>0 && pixels);
   mWidth=width;
   mHeight=height;
   mPixels=pixels;
   setName(name);
   mMipmap=mipmap;
}

void reTexture::setName(char *name)
{
   if(mName) delete [] mName;
   mName=new char[strlen(name)+1];
   strcpy(mName,name);
}

char *reTexture::getName(void) const
{
   return mName;
}

void reTexture::initOGL(void)
{
	
	glGenTextures(1,&mOGLTextureNum);
	glBindTexture(GL_TEXTURE_2D, mOGLTextureNum);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	if(mMipmap)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
//		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		// this scales the image to 2**n times 2**m and builds the mipmap
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, mWidth, mHeight,GL_RGB, GL_UNSIGNED_BYTE, mPixels);
		// use mipmapping filter
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);   
//		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);   
//		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);   
//		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);   
//		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);   
	}
	else
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, mWidth, mHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, mPixels);
	}
}

void reTexture::renderOGL(void)
{
   if(!mOGLTextureNum) initOGL();  // make more efficient later
   // more efficient: do initOGL before rendering starts and then
   // have this assert (and skip the if above):
   //ASSERT(mOGLTextureNum);
   
//   glEnable(GL_TEXTURE_2D);        
   glBindTexture(GL_TEXTURE_2D, mOGLTextureNum);
}


Color3r reTexture::getPixel(Vec2r u) const
{
  Color3r      col;
  unsigned int i;

  i = (unsigned int) (mWidth * (floor( (1-u[1])*mHeight ) + u[0]));

  if (i < 0) i = 0 ;
  if (i >= mWidth*mHeight) i = mWidth*mHeight-1 ;


  col.set( (real) mPixels[3*i] / 255.0, (real) mPixels[3*i+1] / 255.0, (real) mPixels[3*i+2] / 255.0 );

  return col;
}

void reTexture::getPixel(int x,int y, unsigned char &red,unsigned char &green,unsigned char &blue)
{
  ASSERT(mPixels);
  ASSERT(x>=0 && x<=mWidth);
  ASSERT(y>=0 && y<=mHeight);
  red=mPixels[3*(x+mWidth*y)+0];
  green=mPixels[3*(x+mWidth*y)+1];
  blue=mPixels[3*(x+mWidth*y)+2];
}

void reTexture::getPixel(float u,float v, unsigned char &red,unsigned char &green,unsigned char &blue)
{
	int x=int(u*mWidth);
	int y=int(v*mWidth);
	if(!(x>=0 && x<=mWidth)) printf("%f %f\n",u,v);
	ASSERT(mPixels);
	ASSERT(x>=0 && x<=mWidth);
	ASSERT(y>=0 && y<=mHeight);
	red=mPixels[3*(x+mWidth*y)+0];
	green=mPixels[3*(x+mWidth*y)+1];
	blue=mPixels[3*(x+mWidth*y)+2];
}
